////////////////////////////////////////////////////////////////
// Boite de dialogue d'attente d'un client
//
// Sylvain MARECHAL - sylvain.marechal1@libertysurf.fr
////////////////////////////////////////////////////////////////
#include "stdafx.h"
#include "resource.h"
#include "dlgaccept.h"
#include "socket.h"

// #define
#define WM_SOCKET	WM_USER + 1

// Globales
static SOCKET s_hSocket = INVALID_SOCKET;
static SOCKET s_hSocketListen = INVALID_SOCKET;
static int s_nPort;


// Prototypes
static LRESULT CALLBACK WndProcAccept(HWND, UINT, WPARAM, LPARAM);
static BOOL WndProcAcceptInitDialog( HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam);
static BOOL WndProcAcceptCommand( HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam);
static BOOL WndProcAcceptSocket( HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam);


////////////////////////////////////////////////////////////////
// Boite de dialogue d'acceptation de connexion
SOCKET DlgAccept( HINSTANCE hInst, HWND hWnd )
{
    DialogBox(hInst, (LPCTSTR)IDD_ACCEPT, hWnd, (DLGPROC)WndProcAccept);
    return s_hSocket;
}


////////////////////////////////////////////////////////////////
// Procedure de fenetre
LRESULT CALLBACK WndProcAccept(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam)
{
    switch (message)
    {
    case WM_INITDIALOG:
	{
	    return WndProcAcceptInitDialog( hDlg, message, wParam, lParam);
	}
	
    case WM_COMMAND:
	{
	    return WndProcAcceptCommand( hDlg, message, wParam, lParam);
	}

    case WM_SOCKET:
	{
	    return WndProcAcceptSocket( hDlg, message, wParam, lParam);
	}
    }
    return FALSE;
}


////////////////////////////////////////////////////////////////
// Message WM_INITDIALOG
// Initialisation de la boite de dialogue
BOOL WndProcAcceptInitDialog( HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam)
{
    // Remise a 0 des variables
    s_hSocket = INVALID_SOCKET;
    s_hSocketListen = INVALID_SOCKET;

    // Gestion des controles
    if( s_nPort == 0 ) s_nPort = 1234;
    SetDlgItemInt( hDlg, IDC_PORT, s_nPort, FALSE );
    return TRUE;
}


////////////////////////////////////////////////////////////////
// Message WM_COMMAND
// Gestion de l'interaction avec l'utilisateur
BOOL WndProcAcceptCommand( HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam)
{
    switch(wParam)
    {
	case IDCANCEL: // Click sur annuler
	{
	    // On ne veut plus ecouter
	    if( s_hSocket != INVALID_SOCKET ) closesocket( s_hSocket );
	    if( s_hSocketListen != INVALID_SOCKET ) closesocket( s_hSocketListen );
	    
	    // Remise a 0 des variables
	    s_hSocket = INVALID_SOCKET;
	    s_hSocketListen = INVALID_SOCKET;

	    // Sortie de la boite de dlg
	    EndDialog(hDlg, LOWORD(wParam));
	    return TRUE;
	}
	case IDC_ACCEPT:    // Click sur Attente
	{
	    // On commence a ecouter le reseau
	    s_nPort = GetDlgItemInt( hDlg, IDC_PORT, NULL, FALSE );
	    if( s_nPort != 0 )
	    {
		// On cree une socket
		s_hSocketListen = CreateListeningSocket( s_nPort );
		if( s_hSocketListen != INVALID_SOCKET )
		{
		    // On veut recevoir un evenement utilisateur WM_SOCKET
		    // quand une connection se presente
		    if( WSAAsyncSelect( s_hSocketListen, hDlg, WM_SOCKET, FD_ACCEPT ) == SOCKET_ERROR )
		    {
			SocketError( s_hSocketListen, WSAGetLastError() );
			closesocket( s_hSocketListen );
			return -1;
		    }

		    // On grise le bouton et les zones de text
		    EnableWindow( GetDlgItem(hDlg,IDC_PORT), FALSE );
		    EnableWindow( GetDlgItem(hDlg,IDC_ACCEPT), FALSE );
		    return TRUE;
		}				
	    }
	    break;
	}
    }
    return FALSE;
}


////////////////////////////////////////////////////////////////
// WM_SOCKET
// Gestion des evenements reseau
BOOL WndProcAcceptSocket( HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam)
{
    SOCKET hSocket = (SOCKET)wParam; // == s_hSocketListen
    int nError = WSAGETSELECTERROR(lParam);
    int nEvent = WSAGETSELECTEVENT(lParam);
    switch( nEvent )
    {
    case FD_ACCEPT:
	{
	    // On ne veut plus d'evenenement ds cette fenetre
	    WSAAsyncSelect( hSocket, hDlg, 0, 0 );

	    // On accepte la connexion
	    s_hSocket = AcceptSocket( hSocket );
	    if( s_hSocket == INVALID_SOCKET )
	    {
		SocketError( hSocket, nError );
	    }
	    
	    // On ne veut plus ecouter
	    if( hSocket != INVALID_SOCKET ) closesocket( hSocket );

	    // Sortie de la boite de dlg
	    EndDialog(hDlg, LOWORD(wParam));
	    break;
	}
    default:
	{
	    // On ne doit jamais arriver ici
	    break;
	}
    }
    return TRUE;
}
