////////////////////////////////////////////////////////////////
// Boite de dialogue de connexion a un serveur
//
// Sylvain MARECHAL - sylvain.marechal1@libertysurf.fr
////////////////////////////////////////////////////////////////
#include "stdafx.h"
#include "resource.h"
#include "dlgconnect.h"
#include "socket.h"

// #define
#define WM_SOCKET	WM_USER + 1

// Globales
static SOCKET s_hSocket = INVALID_SOCKET;
static char s_szHost[256]; 
static int s_nPort;



// Prototypes
static LRESULT CALLBACK WndProcConnect(HWND, UINT, WPARAM, LPARAM);
static BOOL WndProcConnectInitDialog( HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam);
static BOOL WndProcConnectCommand( HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam);
static BOOL WndProcConnectSocket( HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam);


////////////////////////////////////////////////////////////////
// Boite de dialogue d'acceptation de connexion
SOCKET DlgConnect( HINSTANCE hInst, HWND hWnd )
{
    DialogBox(hInst, (LPCTSTR)IDD_CONNECT, hWnd, (DLGPROC)WndProcConnect);
    return s_hSocket;
}


////////////////////////////////////////////////////////////////
// Procedure de fenetre
LRESULT CALLBACK WndProcConnect(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam)
{
    switch (message)
    {
    case WM_INITDIALOG:
	{
	    return WndProcConnectInitDialog( hDlg, message, wParam, lParam);
	}
	
    case WM_COMMAND:
	{
	    return WndProcConnectCommand( hDlg, message, wParam, lParam);
	}

    case WM_SOCKET:
	{
	    return WndProcConnectSocket( hDlg, message, wParam, lParam);
	}
    }
    return FALSE;
}


////////////////////////////////////////////////////////////////
// Message WM_INITDIALOG
// Initialisation de la boite de dialogue
BOOL WndProcConnectInitDialog( HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam)
{
    // Remise a 0 des variables
    s_hSocket = INVALID_SOCKET;

    // Gestion des controles
    if( s_nPort == 0 ) s_nPort = 1234;
    if( s_szHost[0] == 0 ) strcpy( s_szHost, "localhost" );
    
    SetDlgItemInt( hDlg, IDC_PORT, s_nPort, FALSE );
    SetDlgItemText( hDlg, IDC_HOST, s_szHost );
    return TRUE;
}


////////////////////////////////////////////////////////////////
// Message WM_COMMAND
// Gestion de l'interaction avec l'utilisateur
BOOL WndProcConnectCommand( HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam)
{
    switch(wParam)
    {
	case IDCANCEL: // Click sur annuler
	{
	    // On ne veut plus se connecter
	    if( s_hSocket != INVALID_SOCKET ) closesocket( s_hSocket );
	    
	    // Remise a 0 des variables
	    s_hSocket = INVALID_SOCKET;

	    // Sortie de la boite de dlg
	    EndDialog(hDlg, LOWORD(wParam));
	    return TRUE;
	}
	case IDC_CONNECT:    // Click sur Connexion
	{
	    // On se conecte
	    s_nPort = GetDlgItemInt( hDlg, IDC_PORT, NULL, FALSE );
	    GetDlgItemText( hDlg, IDC_HOST, s_szHost, sizeof(s_szHost) -1 );
	    if( s_nPort != 0 && strlen(s_szHost) > 0 )
	    {
		// On cree une socket
		s_hSocket = CreateSocket();
		if( s_hSocket != INVALID_SOCKET )
		{
		    // On veut recevoir un evenement utilisateur WM_SOCKET
		    // quand la connection est etablie
		    if( WSAAsyncSelect( s_hSocket, hDlg, WM_SOCKET, FD_CONNECT ) == SOCKET_ERROR )
		    {
			SocketError( s_hSocket, WSAGetLastError() );
			closesocket( s_hSocket );
			return -1;
		    }

		    // On se connecte (mode non bloquant)
		    if( ConnectSocket( s_hSocket, s_szHost, s_nPort ) == SOCKET_ERROR )
		    {
			// En mode non bloquant, on sort toujours en erreur
			// WSAEWOULDBLOCK lors d'une tentative de connexion
			if( WSAGetLastError() != WSAEWOULDBLOCK )
			{
			    SocketError( s_hSocket, WSAGetLastError() );
			    closesocket( s_hSocket );
			    return -1;
			}
		    }

		    // On grise le bouton et les zones de text
		    EnableWindow( GetDlgItem(hDlg,IDC_PORT), FALSE );
		    EnableWindow( GetDlgItem(hDlg,IDC_HOST), FALSE );
		    EnableWindow( GetDlgItem(hDlg,IDC_CONNECT), FALSE );
		    return TRUE;
		}				
	    }
	    break;
	}
    }
    return FALSE;
}


////////////////////////////////////////////////////////////////
// WM_SOCKET
// Gestion des evenements reseau
BOOL WndProcConnectSocket( HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam)
{
    SOCKET hSocket = (SOCKET)wParam; // == s_hSocket
    int nError = WSAGETSELECTERROR(lParam);
    int nEvent = WSAGETSELECTEVENT(lParam);
    switch( nEvent )
    {
    case FD_CONNECT:
	{
	    // On ne veut plus d'evenenement ds cette fenetre
	    WSAAsyncSelect( hSocket, hDlg, 0, 0 );
	    
	    // Connexion etablie ou erreur
	    if( nError != 0 )
	    {
		SocketError( hSocket, nError );
		closesocket( hSocket );
		s_hSocket = INVALID_SOCKET;
	    }

	    // Sortie de la boite de dlg
	    EndDialog(hDlg, LOWORD(wParam));
	    break;
	}
    default:
	{
	    // On ne doit jamais arriver ici
	    break;
	}
    }
    return TRUE;
}
