////////////////////////////////////////////////////////////////
// Programme principal pour :
//  -Turbo C
//  -Win32 console
//  -unix (enfin, au moins linux)
//
// Sylvain MARECHAL - sylvain.marechal1@libertysurf.fr
////////////////////////////////////////////////////////////////
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <time.h>
#include <ctype.h>
#include "pongcore.h"
#include "pongdp.h"
#include "pongkey.h"
#include "pongkevt.h"
#include "pongtevt.h"

#ifndef __TURBOC__ // Pas de socket avec turboc
#include "pongsevt.h"
#include "waitevt.h"
#include "pongsock.h"
#endif //!__TURBOC__

#if ! defined(_WIN32) && ! defined(__TURBOC__)
#include "kbu.h"
#include "coniounix.h"
#endif
#ifdef _WIN32
#include "coniowin32.h"
#endif
#ifdef __TURBOC__
#include <conio.h>
#endif 

// Taille de l'ecran
#ifdef _WIN32
#define	LIGNES	50
#else
#define	LIGNES	25
#endif

// Proto statics
static void PongPlay();
static void PongSockPlay();
static void InitKeyboard();
static void RestoreKeyboard();


////////////////////////////////////////////////
// main()
////////////////////////////////////////////////
int main()
{
    // Initialisation du clavier (pour unix) et curseur
    InitKeyboard();

    // Intialise les globales
    // 80 colonnes, 25/50 lignes, dim raquette de x=1 par y=4, dim balle de x=y=1
    InitPongGlobals(80, LIGNES, 1, 4, 1);
    
    do
    {
	// Desine le pong
	InitPongDisplay();
	
	// Nouvelle partie ?
	PongStatusDisplay( "Nouvelle partie O/N ?" );
	int key = PongWaitKeys( "oONn", 4 );
	if( toupper(key) != 'O' )
	{
	    break;
	}
	
	// Go
#ifdef __TURBOC__
	// Mode joueur seul pour turbo C
	PongPlay();
#else
	// Mode joueur seul / client /server pour win32/unix
	PongSockPlay();
#endif
    } while( 1 );

    // Restauration du clavier et curseur
    RestoreKeyboard();
    return 0;
}

////////////////////////////////////////////////
// PongPlay()
////////////////////////////////////////////////
void PongPlay()
{
    int fExit = 0;
    int Speed = 1;	    
    
    // Remet la balle et la raquette 
    PongPlayReset();

    // Indique comment sortir
    PongStatusDisplay( "Q pour quitter" );
    
    // Vitesse par defaut
    PongSetSpeed( Speed );
    
    while(!fExit)
    {
	char aKeys[] = { FLECHE_HAUT, FLECHE_BAS, FLECHE_GAUCHE, FLECHE_DROITE, 'q', 'Q' };

	// Extraction d'une touche faisant partie du tableau
	// sor au bout de 100 millisecondes
	int KeyOrBallMove = PongWaitOrExtractKey(aKeys, 6);
	
	// PongManageKeyboardEvent() renvoie 0 si la touche n'a
	// pas ete traite (=> ce n'est pas une touche
	// renvoie -3 si on veut quitter (Q)
	int key = PongManageKeyboardEvent( KeyOrBallMove, PONG_ALONE );
	if( key == 0 )
	{
	    // La balle bouge
	    GameStatus Status = PongManageTimeoutEvent( PONG_ALONE );
	    if( Status == PERDU )
	    {
		PongStatusDisplay( "Perdu !!" );
		PongWaitKeys( "oONnQq ", 7 );
		return;
	    }
	}
	else if( toupper(key) == 'Q' )
	{
	    fExit = 1;
	}
    }
}

#ifndef __TURBOC__
////////////////////////////////////////////////
// PongSockConnect()
////////////////////////////////////////////////
static SOCKET PongSockConnect()
{
    SOCKET hSocket;
    char szHost[256];
    int nPort = 1234;

    // Saisie du host
    PongStatusGetString( "Rentrez le nom de la machine (port=1234)", szHost, sizeof(szHost) );

    // Tentative de connexion
    hSocket = CreateSocket();
    if( ConnectSocket( hSocket, szHost, nPort ) == SOCKET_ERROR )
    {
	closesocket( hSocket );
	return INVALID_SOCKET;
    }

    // Tout semble OK
    return hSocket;
}

////////////////////////////////////////////////
// PongSockAccept()
////////////////////////////////////////////////
static SOCKET PongSockAccept()
{
    SOCKET hListeningSocket, hSocket;
    int nPort = 1234;

    // Message
    PongStatusDisplay( "Attente d'un client (port=1234)" );

    // Attente d'une connexion
    hListeningSocket = CreateListeningSocket( nPort );
    hSocket = AcceptSocket( hListeningSocket );
    if( hSocket == INVALID_SOCKET )
    {
	closesocket(hListeningSocket); 
	closesocket( hSocket );
	return INVALID_SOCKET;
    }
    
    // Tout semble OK
    closesocket(hListeningSocket); 
    return hSocket;
}

////////////////////////////////////////////////
// PongSockAccept()
////////////////////////////////////////////////
static SOCKET PongPlayClientServer( PongMode * pMode )
{
    PongStatusDisplay( "Client / Serveur / Joueur seul ? (C/S/J)" );
    int key = PongWaitKeys( "cCsSjJnNOo  ", 12 );
    if( toupper(key) == 'C' )
    {
	*pMode = PONG_CLIENT;
	return PongSockConnect();
    }
    else if( toupper(key) == 'S' )
    {
	*pMode = PONG_SERVER;
	return PongSockAccept();
    }
    *pMode = PONG_ALONE;
    return INVALID_SOCKET;
}




////////////////////////////////////////////////
// PongSockPlay()
////////////////////////////////////////////////
void PongSockPlay()
{
    int ret = 0, key = 0;;
    int Speed = 1;	    
    char aKeys[] = { FLECHE_HAUT, FLECHE_BAS, FLECHE_GAUCHE, FLECHE_DROITE, 'q', 'Q' };
    EvtRecord aEvt[sizeof(aKeys)/sizeof(char) + 1 + 1]; // 6 touches, 1 event reseau, 1 timer
    int aEventsSet[sizeof(aKeys)/sizeof(char) + 1 + 1];
    int cbEventsSet;
    int cbEvt = 0;
    PongMode Mode;
    SOCKET hSocket;

    // Recuperation socket pour les modes client/server
    hSocket = PongPlayClientServer(&Mode);
    if( hSocket == INVALID_SOCKET && Mode != PONG_ALONE )
    {
	PongStatusDisplay( "Probleme de connexion !" );
	PongWaitKeys( "oONnQq ", 7 );
	return;
    }

    // On commence par initialiser le tableau d'evts
    // Touches gerees
    for( cbEvt = 0; cbEvt < sizeof(aKeys)/sizeof(char); cbEvt ++ )
    {
	aEvt[cbEvt].EvtType = EV_TYPE_KEYBOARD;
	aEvt[cbEvt].EvtKeyboard.AsciiChar = aKeys[cbEvt];
	aEvt[cbEvt].EvtKeyboard.ScanCode = aKeys[cbEvt];
    }
  
    // Evenement reseau
    if( Mode != PONG_ALONE )
    {
	aEvt[cbEvt].EvtType = EV_TYPE_SOCKET;
	aEvt[cbEvt].EvtSocket.hSocket = hSocket;
	aEvt[cbEvt].EvtSocket.MaskIn = FD_READ;
	cbEvt ++;
    }

    // Timer
    aEvt[cbEvt].EvtType = EV_TYPE_TIME;
    aEvt[cbEvt].EvtTime.timeout = 100;
    cbEvt ++;

    // On envoie les dimensions de notre ecran
    if( Mode != PONG_ALONE )
    {
	PongSocketInitializeSocket( hSocket );
	PongSocketSendDims();
    }

    // Initialisation des events
    InitExtractEvent( aEvt, cbEvt );
        
    // Remet la balle et la raquette 
    PongPlayReset();

    // Indique comment sortir
    PongStatusDisplay( "Q pour quitter" );
    
    // Vitesse par defaut
    PongSetSpeed( Speed );

    
    // On boucle tant que ca joue
    while( ret >= 0 && toupper(key) != 'Q' )
    {
	GameStatus Status;
	
	// Extraction des evenements
	ret = ExtractEvents( aEvt, cbEvt, aEventsSet, &cbEventsSet  );
	
	// On boucle sur tous les evenements extraits qui sont dans le 
	// tableau aEventsSet[]. Ce tableau contient l'index de l'evenement
	// leve
	for( int i = 0; i < cbEventsSet; i ++ )
	{
	    int iEvent = aEventsSet[i];

	    switch( aEvt[iEvent].EvtType )
	    {
	    case EV_TYPE_KEYBOARD :
		{
		    // On a appuye sur une touche reconnue
		    key = PongManageKeyboardEvent( aEvt[iEvent].EvtKeyboard.AsciiChar, Mode );
		    break;
		}
	    case EV_TYPE_TIME :
		{
		    // Il est temps de faire bouger la balle
		    Status = PongManageTimeoutEvent( Mode );
		    break;
		}
	    case EV_TYPE_SOCKET:
		{
		    // Y'a qqchose ka bouge chez le client/serveur
		    ret = PongManageSocketEvent( aEvt[iEvent].EvtSocket.hSocket, 
					aEvt[iEvent].EvtSocket.MaskOut, 
					&aEvt[iEvent].EvtSocket.MaskIn, &Status );
		    if( ret < 0 )
		    {
			PongStatusDisplay( "Deconnexion" );
			PongWaitKeys( "oONnqQ ", 7 );
		    }
		    break;
		}
	    }
	}

	// Pourquoi le jeu s'est-il arrete ?
	switch(Status)
	{
	case PERDU:
	    {
		if( Mode == PONG_SERVER )
		{
		    PongStatusDisplay( "Perdu !! Nouvelle partie O/N ?" );
		    int touche = PongWaitKeys( "oONn", 4 );
		    if( toupper(touche) == 'N' ) key = 'Q';
		}
		else
		{
		    PongStatusDisplay( "Perdu !!" );
		    sleep(5);
		}
		InitPongDisplay();
		PongStatusDisplay( "Q pour quitter" );
		PongPlayReset();
		break;
	    }
	case GAGNE:
	    {
		if( Mode == PONG_SERVER )
		{
		    PongStatusDisplay( "Gagne !! Nouvelle partie O/N ?" );
		    int touche = PongWaitKeys( "oONn", 4 );
		    ret = 1;
		    if( toupper(touche) == 'N' ) key = 'Q';
		}
		else
		{
		    PongStatusDisplay( "Gagne !!" );
		    sleep(5);
		}
		InitPongDisplay();
		PongStatusDisplay( "Q pour quitter" );
		PongPlayReset();
		break;
	    }
	}
    }
    
    // On fait du nettoyage avant de sortir
    if( hSocket != INVALID_SOCKET ) 
    {
	closesocket( hSocket );
	PongSocketDestroySocket();
    }
    CloseExtractEvent( aEvt, cbEvt );

}
#endif ! __TURBOC__




void InitKeyboard()
{
#if ! defined(_WIN32) && ! defined(__TURBOC__)
    init_keyboard();
#endif
    _setcursortype( _NOCURSOR );
}

void RestoreKeyboard()
{
#if ! defined(_WIN32) && ! defined(__TURBOC__)
    close_keyboard();
#endif
    _setcursortype( _NORMALCURSOR );
}

