////////////////////////////////////////////////////////////////
// Ce module contient le coeur du jeu
// Gestion de la trajectoire de la balle et des raquettes
// ainsi que des collisions avec les murs
//
// Sylvain MARECHAL - sylvain.marechal1@libertysurf.fr
////////////////////////////////////////////////////////////////
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <time.h>
#include "pongcore.h"
#include "pongdp.h"


// Variables globales
// Variables constantes
static int s_xLargeur;	    // Largeur de l'ecran
static int s_yHauteur;	    // Hauteur de l'ecran
static int s_xBatLen;	    // Largeur de la raquette
static int s_yBatLen;	    // Hauteur de la raquette
static int s_xyBallLen;	    // Diametre de la balle

// Variable de positions
static int s_xBallPos = 0;  // Position de la balle en x
static int s_yBallPos = 0;  // Position de la balle en y
static int s_yBatPosL = 0;  // Position de la raquette gauche
static int s_yBatPosR = 0;  // Position de la raquette droite

// Pas d'avancement de la balle
static int s_xBallMove = 1;


	    

////////////////////////////////////////////////
// InitPongGlobals()
//  Initialise les globales necessaires a la 
//  gestion de l'ecran 
////////////////////////////////////////////////
void InitPongGlobals( int xLargeur, int yHauteur, int xBatLen, int yBatLen, int xyBallLen )
{
    // Sauve les globales
    s_xLargeur = xLargeur;
    s_yHauteur = yHauteur;
    s_xBatLen = xBatLen;
    s_yBatLen = yBatLen;
    s_xyBallLen = xyBallLen;
}

////////////////////////////////////////////////
// PongSetSpeed()
//  initialise la rapidite
////////////////////////////////////////////////
void PongSetSpeed( int xBallMove )
{
    if( s_xBallMove <= 0 ) s_xBallMove = 1;
    s_xBallMove = xBallMove;
}

////////////////////////////////////////////////
// PongSetSpeed()
//  renvoie la rapidite
////////////////////////////////////////////////
int PongGetSpeed()
{
    return s_xBallMove;
}

////////////////////////////////////////////////
// PongGetLargeur()
//  Renvoie la largeur de l'ecran 
////////////////////////////////////////////////
int PongGetLargeur() 
{ 
    return s_xLargeur; 
}

////////////////////////////////////////////////
// PongGetLargeur()
//  Renvoie la hauteur de l'ecran 
////////////////////////////////////////////////
int PongGetHauteur()
{
    return s_yHauteur;
}

////////////////////////////////////////////////
// PongGetXBallPos()
//  Renvoie la position x de la balle
////////////////////////////////////////////////
int PongGetXBallPos()
{
    return s_xBallPos;
}

////////////////////////////////////////////////
// PongGetYBallPos()
//  Renvoie la position y de la balle
////////////////////////////////////////////////
int PongGetYBallPos()
{
    return s_yBallPos;
}

////////////////////////////////////////////////
// PongGetLeftBatPos()
//  Renvoie la position y de la raquette gauche
////////////////////////////////////////////////
int PongGetLeftBatPos()
{
    return s_yBatPosL;
}

////////////////////////////////////////////////
// PongGetRightBatPos()
//  Renvoie la position y de la raquette droite
////////////////////////////////////////////////
int PongGetRightBatPos()
{
    return s_yBatPosR;
}

////////////////////////////////////////////////
// PongGetLargeur()
//  Renvoie la largeur de la raquette
////////////////////////////////////////////////
int PongGetBatLen()
{
    return s_yBatLen;
}

////////////////////////////////////////////////
// PongPlayReset()
////////////////////////////////////////////////
void PongPlayReset()
{
    // Reinitialize tout
    s_xBallPos = 0;
    s_yBallPos = 0;
    PongLeftBatMove( 0 );
    PongRightBatMove( 0 );
}

////////////////////////////////////////////////
// PongSetBatPos()
//  Gestion du mouvement de la raquette
////////////////////////////////////////////////
static void PongSetBatPos( int yPos, int xBatPos, int fLeft )
{
    int yBatPrec, yBatPos;


    // Sauvegarde ancienne position
    yBatPos = ( fLeft ? s_yBatPosL : s_yBatPosR);
    yBatPrec = yBatPos;

    // On avance de decal
    yBatPos = yPos;
    
    // Gestion des colisions
    if( yBatPos < 2 ) yBatPos = 2;
    if( yBatPos > s_yHauteur - 2 - s_yBatLen ) yBatPos = s_yHauteur - 2 - s_yBatLen;

    // Affiche la raquette
    PongBatDisplay( yBatPrec, xBatPos, yBatPos, s_xBatLen, s_yBatLen, fLeft );

    // Sauve la nouvelle position
    if( fLeft )
    {
	s_yBatPosL = yBatPos;
    }
    else
    {
	s_yBatPosR = yBatPos;
    }
}

////////////////////////////////////////////////
// PongBatMove()
//  Gestion du mouvement de la raquette
////////////////////////////////////////////////
static void PongBatMove( int Decal, int xBatPos, int fLeft )
{
    int yBatPos;

    // Gestion premiere fois
    if( s_yBatPosL == 0 )
    {
	s_yBatPosL = (s_yHauteur - 1  - s_yBatLen - 3) / 2;
    }
    if( s_yBatPosR == 0 )
    {
	s_yBatPosR = (s_yHauteur - 1  - s_yBatLen - 3) / 2;
    }
    yBatPos = ( fLeft ? s_yBatPosL : s_yBatPosR);

    // On avance de decal
    yBatPos += Decal;

    // On affiche en verifiant qu'on ne deborde pas
    // de l'ecran
    PongSetBatPos( yBatPos, xBatPos, fLeft );
}

////////////////////////////////////////////////
// PongLeftBatMove()
//  Gestion du mouvement de la raquette gauche
////////////////////////////////////////////////
void PongLeftBatMove( int Decal )
{
    PongBatMove( Decal, 2+s_xBatLen, 1 );
}

////////////////////////////////////////////////
// PongRightBatMove()
//  Gestion du mouvement de la raquette droite
////////////////////////////////////////////////
void PongRightBatMove( int Decal )
{
    PongBatMove( Decal, s_xLargeur - 2*s_xBatLen, 0 );
}

////////////////////////////////////////////////
// PongSetLeftBatPos()
//  Gestion du mouvement de la raquette gauche
//  Utilisation en reseau
////////////////////////////////////////////////
void PongSetLeftBatPos( int yPos)
{
    PongSetBatPos( yPos, 2+s_xBatLen, 1 );
}

////////////////////////////////////////////////
// PongSetRightBatPos()
//  Gestion du mouvement de la raquette droite
//  Utilisation en reseau
////////////////////////////////////////////////
void PongSetRightBatPos( int yPos)
{
    PongSetBatPos( yPos, s_xLargeur - 2*s_xBatLen, 0 );
}

////////////////////////////////////////////////
// PongInitRand()
//  Initialise le hasard pour qu'il soit 
//  hasardeux
////////////////////////////////////////////////
static void PongInitRand()
{
    static int s_first = 0;

    if( ! s_first )
    {
	s_first = 1;
	srand( (unsigned)time( NULL ) );
    }
}

////////////////////////////////////////////////
// PongGetRandSigne()
//  Renvoie -1 ou +1 au hasard
////////////////////////////////////////////////
static int PongGetRandSigne()
{
    PongInitRand();
    return rand() % 100 > 50 ? -1 : 1;
}

////////////////////////////////////////////////
// PongGetRandCoef()
//  Calcule un coeficient directeur de la 
//  droite au hasard
////////////////////////////////////////////////
static int PongGetRandCoef()
{
    return rand() % 100;
}

////////////////////////////////////////////////
// PongGetNextY()
//  Calcule la prochaine position
//  avec la formule y = (x -x0)*coef + y0
////////////////////////////////////////////////
static int PongGetNextY( int x0, int y0, int xPos, int coef )
{
    int yPos = (xPos-x0 ) * coef / 100 + y0;
    return yPos;
}

////////////////////////////////////////////////
// PongBallMove()
//  Gestion du deplacement de la balle.
//  On change de direction des qu'on rencontre
//  un mur.
//  On renvoie l'etat du jeu suite au deplacement
//  cad, GAGNE/PERDU ou ENCOURS
////////////////////////////////////////////////
GameStatus PongBallMove()
{
    static int s_Coef, s_Signe, s_Sens;
    static int s_x0, s_y0;
    GameStatus Status = ENCOURS;

    
    // Gestion premiere fois
    if( s_xBallPos == 0 && s_yBallPos == 0 )
    {
	s_xBallPos = 3 + s_xBatLen;
	s_yBallPos = s_yBatPosL + s_yBatLen/2;
	s_Signe = PongGetRandSigne();
	s_Coef = PongGetRandCoef() * s_Signe;
	s_Sens = 1;
	s_x0 = s_xBallPos;
	s_y0 = s_yBallPos;
    }
    
    // Sauvegarde l'ancienne position
    int xPrec = s_xBallPos;
    int yPrec = s_yBallPos;

    // Calcule la nouvelle position
    s_xBallPos += (s_Sens*s_xBallMove);
    s_yBallPos = PongGetNextY( s_x0, s_y0, s_xBallPos, s_Coef );


    // Tests de colision avec un mur
    int fRecalcCoef = 0;
    if( s_xBallPos < 3 + s_xBatLen && s_xBallPos != s_x0 )
    {
	// La balle revient vers nous
	// On regarde si la raquette est bien placee
	if( s_yBallPos >= s_yBatPosL && s_yBallPos <= s_yBatPosL + s_yBatLen )
	{
	    s_Sens = 1;
	    fRecalcCoef = 1;
	    s_xBallPos = 3 + s_xBatLen;
	}
	else
	{
	    // Perdu !
	    Status = PERDU; // On se place du point de vu du serveur
	}
    }
    else if( s_xBallPos + (s_xyBallLen-1) > s_xLargeur - 3 - s_xBatLen && s_xBallPos != s_x0 )
    {
	// La balle tape au fond
	// On regarde si la raquette est bien placee
	if( s_yBallPos >= s_yBatPosR && s_yBallPos <= s_yBatPosR + s_yBatLen )
	{
	    fRecalcCoef = 1;
	    s_xBallPos = s_xLargeur - s_xyBallLen - 2*s_xBatLen;
	    s_Sens = -1;
	}
	else
	{
	    // Gagne !
	    Status = GAGNE; // On se place du point de vu du serveur
	}
    }
    else if( s_yBallPos < 2 && s_yBallPos != s_y0 )
    {
	// La balle tape en haut
	fRecalcCoef = 1;
	s_yBallPos = 2;
    }
    else if( s_yBallPos  + (s_xyBallLen-1) > s_yHauteur - 3 && s_yBallPos != s_y0 )
    {
	// La balle tape en bas
	fRecalcCoef = 1;
	s_yBallPos = s_yHauteur - 3 - (s_xyBallLen-1);
    }

    
    // On a heurte un bord, on change de direction
    if( fRecalcCoef )
    {
	s_Signe = (s_Signe == 1 ? -1 : 1);
	s_Coef = PongGetRandCoef() * s_Signe;
	s_x0 = s_xBallPos;
	s_y0 = s_yBallPos;
    }

    // On affiche la balle a sa nouvelle position
    PongBallDisplay( xPrec, yPrec, s_xBallPos, s_yBallPos, s_xyBallLen );
    return Status;
}

////////////////////////////////////////////////
// PongSetBallPos()
//  Utilisation en reseau
//  Affiche la balle a l'endroit indique
//  La fonction retouche les parametres de maniere
//  que la balle soit toujours dans l'ecran
////////////////////////////////////////////////
void PongSetBallPos( int x, int y )
{
    // Sauvegarde l'ancienne position
    int xPrec = s_xBallPos;
    int yPrec = s_yBallPos;

    // Gestion reinit
    if( x == 0 && x == 0 )
    {
	x = 3 + s_xBatLen;
	x = s_yBatPosL + s_yBatLen/2;
    }

    // On recadre les positions au cas ou ca deborde
    if( x < 3 + s_xBatLen )
    {
	x = s_xBatLen + 3;
    }
    if( x + (s_xyBallLen-1) > s_xLargeur - 3 - s_xBatLen )
    {
	x = s_xLargeur - s_xyBallLen - 2*s_xBatLen;
    }
    if( y < 2 )
    {
	y = 2;
    }
    if( y + (s_xyBallLen-1) > s_yHauteur -3 )
    {
	y = s_yHauteur - 3 - (s_xyBallLen-1);
    }
    
    // Nouvelle position
    s_xBallPos = x;
    s_yBallPos = y;
    
    // On affiche la balle a sa nouvelle position
    PongBallDisplay( xPrec, yPrec, s_xBallPos, s_yBallPos, s_xyBallLen );
}

