////////////////////////////////////////////////////////////////
// Gestion des affichages sous :
// - win32 console, 
// - turbo c++ 3.0
// - unix
// Repose sur les fonctions gotoxy() et clrscr()
//
// Sylvain MARECHAL - sylvain.marechal1@libertysurf.fr
////////////////////////////////////////////////////////////////
#include <stdio.h>
#include <string.h>
#include "pongdp.h"
#include "pongcore.h"
#ifdef _WIN32
#include <conio.h>
#include "coniowin32.h"
#elif defined __TURBOC__
#include <conio.h>
#else
#include "coniounix.h"
#endif

#ifndef min
#define min(a,b) ((a) < (b)) ? (a) : (b)
#endif


////////////////////////////////////////////////
// putchardu()
//  Ajoute un '\n' sous unix afin que l'affichage
//  soit immediat
////////////////////////////////////////////////
static void putchardu( int c )
{
    putchar( c );
#if !defined(_WIN32) && !defined(__TURBOC__)
    putchar( '\n' );
#endif
}

////////////////////////////////////////////////
// InitPongDisplay()
////////////////////////////////////////////////
void InitPongDisplay()
{
    // Efface tout
    clrscr();
    
    // Dessine le tour
    // Les lignes
    for( int x = 1; x < PongGetLargeur(); x ++ )
    {
	// Premiere ligne
	gotoxy( x, 1 );			    putchar( 176 );
	
	// Avant la ligne de status
	gotoxy( x, PongGetHauteur() - 2 );  putchar( 176 );
	
	// Apres la ligne de status
	gotoxy( x, PongGetHauteur() );	    putchar( 176 );
    }
    
    // Les colonnes
    for( int y = 1; y < PongGetHauteur(); y ++ )
    {
	// Premiere colonne
	gotoxy( 1, y );			    putchar( 176 );
	
	// Derniere colonne
	gotoxy( PongGetLargeur(), y );	    putchar( 176 );
    }
}

////////////////////////////////////////////////
// PongStatusDisplay()
////////////////////////////////////////////////
void PongStatusDisplay( char * pszStatus )
{
    // Commence par effacer
    for( int x = 2; x < PongGetLargeur(); x ++ )
    {
	gotoxy( x, PongGetHauteur() - 1 );
	putchar( ' ' );
    }
    
    // Ecrit la ligne de status en bas
    if( pszStatus )
    {
	int cbStatus = strlen(pszStatus);
	if( cbStatus < PongGetLargeur() - 2 )
	{
	    int x = (PongGetLargeur() - cbStatus)/2;
	    gotoxy( x, PongGetHauteur() - 1 );
	    printf( "%s\n", pszStatus );
	}
    }
}

////////////////////////////////////////////////
// PongStatusGetString()
////////////////////////////////////////////////
int PongStatusGetString( char * pszMsg, char * pszString, int /*cbString*/ )
{
    PongStatusDisplay( "" );
    gotoxy( 3, PongGetHauteur() - 1 );
    printf( "%s\n", pszMsg );
    gotoxy( 4 + strlen(pszMsg), PongGetHauteur() - 1 );
    return scanf( "%s", pszString );
}

////////////////////////////////////////////////
// PongBatDisplay()
// Affiche la raquette
////////////////////////////////////////////////
void PongBatDisplay( int yBatPrec, int xBatPos, int yBatPos, int xBatLen, int yBatLen, int /*fLeft*/ )
{
    // Boucle sur l'epaisseur de la raquette (a priori 1 en mode texte)
    for( int x = xBatPos; x < xBatPos + xBatLen; x ++ )
    {
	// Commence par effacer
	int y, yMin;
	yMin = min(yBatPrec + yBatLen, PongGetHauteur() - 2);
	for( y = yBatPrec; y < yMin; y ++ )
	{
	    gotoxy( x, y );
	    putchardu( ' ' );
	}

	// Affiche la raquette
	for( y = yBatPos; y < yBatPos + yBatLen; y ++ )
	{
	    gotoxy( x, y );
	    putchardu( 219 );
	}
    }
}

////////////////////////////////////////////////
// PongBallDisplay()
// Affiche la balle
////////////////////////////////////////////////
void PongBallDisplay( int xPrec, int yPrec, int xBallPos, int yBallPos, int xyBallLen )
{
    int x, y;

    // On boucle sur la dimension de la balle (a priori 1 en mode texte)
    // Efface l'ancienne pos
    for( x = xPrec; x < xPrec + xyBallLen; x ++ )
    {
	for( y = yPrec; y < yPrec + xyBallLen; y ++ )
	{
	    gotoxy( x, y );
	    putchardu( ' ' );
	}
    }
    
    // On boucle sur la dimension de la balle (a priori 1 en mode texte)
    // Affiche la balle
    for( x = xBallPos; x < xBallPos + xyBallLen; x ++ )
    {
	for( y = yBallPos; y < yBallPos + xyBallLen; y ++ )
	{
	    gotoxy( x, y );
#if !defined(_WIN32) && !defined(__TURBOC__)
	    putchardu( 'O' );
#else
	    putchardu( 2 );
#endif
	}
    }
}

