////////////////////////////////////////////////////////////////
// Gestion du clavier pour les mode console
//  Utilise par Turbo C
//
// Sylvain MARECHAL - sylvain.marechal1@libertysurf.fr
////////////////////////////////////////////////////////////////
#include <stdio.h>
#if defined(_WIN32) || defined(__TURBOC__)
#include <conio.h>
#else
#include "kbu.h"
#endif
#include <time.h>
#include "pongkey.h"

#ifndef CLOCKS_PER_SEC
#define CLOCKS_PER_SEC	CLK_TCK 
#endif


////////////////////////////////////////////////
// PongWaitKeys()
// - Retourne uniquement si la touche pressee
//   par l'utilisateur fait partie du tableau
//
////////////////////////////////////////////////
int PongWaitKeys( char * aKeys, int cbKeys )
{
    while(1)
    {
	int key = getch();
	for( int i = 0; i < cbKeys; i ++ )
	{
	    if( key == aKeys[i] ) return key;
	}
    }
#ifndef __TURBOC__ // Supprime le warning
    return 0; // Jamais atteint
#endif
}

////////////////////////////////////////////////
// PongWaitOrExtractKey()
// - Renvoie la touche pressee si elle fait partie
//   du tableau passe en arguments
// - Renvoie 0 si le timeout (100 ms) est ecoule
//
////////////////////////////////////////////////
int PongWaitOrExtractKey( char * aKeys, int cbKeys )
{
    static clock_t s_LastClockBallMove = 0;
    const clock_t DelayBallMove = CLOCKS_PER_SEC/10;
    
    
    while( 1 )
    {
	// On extrait une touche si y'en a une en attente
	int key = 0;
	while( kbhit() != 0 )
	{
	    key = getch();
	}

	// Si la touche fait partie du tableau de touches
	// alors on sort
	for( int i = 0; i < cbKeys; i ++ )
	{
	    if( key == aKeys[i] ) 
	    {
		// Touche reconnue pressee
		return key;
	    }
	}
	
	// On ne sort que si le timeout est ecoule
	clock_t CurrentClockBallMove = clock();
	if( CurrentClockBallMove - s_LastClockBallMove >= DelayBallMove )
	{
	    // Timeout ecoule
	    s_LastClockBallMove = CurrentClockBallMove;
	    return 0;
	}    
    }
    
    // Jamais atteint
#ifndef __TURBOC__ // Supprime le warning
    return -1;
#endif
}

