// Gestion de socket bloquantes/non bloquantes
//
// Sylvain MARECHAL - sylvain.marechal1@libertysurf.fr
////////////////////////////////////////////////////////////////
#include <stdio.h>
#include "socket.h"

////////////////////////////////////////////////////////////////
void InitializeSocketLib()
{
#ifdef _WIN32
    // Initialisation de Winsock
    WSADATA wsaData;
    WSAStartup( MAKEWORD( 2, 2 ), &wsaData );
#endif
}


////////////////////////////////////////////////////////////////
void SocketError( SOCKET hSocket, int error )
{
    char Buffer[256];

    // Message
    sprintf( Buffer, "Erreur de socket %d", error );
#ifdef _WIN32
    MessageBox( NULL, Buffer, "Erreur", MB_ICONEXCLAMATION + MB_TASKMODAL );
#else
    printf( "%s\n", Buffer );
#endif
}


////////////////////////////////////////////////////////////////
SOCKET CreateListeningSocket( int nPort )
{
    SOCKET hSocket;

    // Initialisation
    InitializeSocketLib();

    // On cree une socket
    hSocket = socket( PF_INET, SOCK_STREAM, 0 );
    if( hSocket == INVALID_SOCKET )
    {
	SocketError( hSocket, SOCKETERRNO );
	closesocket( hSocket );
	return INVALID_SOCKET;
    }
    
    struct sockaddr_in addr;
    int len = sizeof(addr);
    
    // Liaison avec le port porLocal
    addr.sin_family = AF_INET ;
    addr.sin_addr.s_addr = htonl (INADDR_ANY);
    addr.sin_port = htons ((unsigned short)nPort );
    if( bind( hSocket, (struct sockaddr *)&addr, sizeof(addr)) == SOCKET_ERROR )
    {
	SocketError( hSocket, SOCKETERRNO );
	closesocket( hSocket );
	return INVALID_SOCKET;
    }

    // On indique que cette socket va ecouter
    if( listen( hSocket, 1) == SOCKET_ERROR )
    {
	SocketError( hSocket, SOCKETERRNO );
	closesocket( hSocket );
	return INVALID_SOCKET;
    }

    return hSocket;
}


////////////////////////////////////////////////////////////////
SOCKET CreateSocket()
{
    SOCKET hSocket;

    // Initialisation
    InitializeSocketLib();

    // On cree une socket
    hSocket = socket( PF_INET, SOCK_STREAM, 0 );
    if( hSocket == INVALID_SOCKET )
    {
	SocketError( hSocket, SOCKETERRNO );
	closesocket( hSocket );
	return INVALID_SOCKET;
    }
    
    return hSocket;
}


////////////////////////////////////////////////////////////////
struct sockaddr SocketFormatAdress( char * host, u_short port )
{
    struct sockaddr_in addr;
    struct sockaddr addrRet;
    struct hostent *lphost ;
    u_long IP;
    
    
    memset((char*)&addr, 0, sizeof(addr));
    /*	Soit on fournit une adresse IP, soit on fournit un nom	*/
    if ((IP = inet_addr(host)) == (u_long)INADDR_NONE)
    {
	if ((lphost  = gethostbyname(host))==NULL)
	{
            memset( (char * )&addrRet, 0, sizeof(addrRet) );
	    return  addrRet;
	}
	addr.sin_family = lphost->h_addrtype;
#ifdef _WIN16 /* A dfinir dans le projet WIN16	*/
	_fmemcpy (&addr.sin_addr, lphost->h_addr, lphost->h_length);
#else /*	WIN32, UNIX*/
	memcpy (&addr.sin_addr, lphost->h_addr, lphost->h_length);
#endif
    }
    else
    {
        addr.sin_family = AF_INET;
        addr.sin_addr.s_addr = IP;
    }
    
    /*  Port destination    */    
    addr.sin_port = htons((u_short)port );    
    
    memcpy( (char *)&addrRet, (char *)&addr, sizeof(addrRet) );
    return addrRet;
}


////////////////////////////////////////////////////////////////
int ConnectSocket( SOCKET hSocket, char * pszHost, int nPort )
{
    struct sockaddr ConnectAddress;
    
    // On transforme l'adresse pour les sockets
    ConnectAddress = SocketFormatAdress( pszHost, (u_short )nPort );

    // On se connecte
    return connect( hSocket, &ConnectAddress, sizeof(ConnectAddress) );
}


////////////////////////////////////////////////////////////////
SOCKET AcceptSocket( SOCKET  hListeningSocket )
{
    SOCKET hSocket = accept( hListeningSocket, NULL, NULL );
    return hSocket;
}


////////////////////////////////////////////////////////////////
int SetBlockingModeSocket( SOCKET hSocket, int fBlocking )
{
    int r;
#ifdef _WIN32
    {
	int sock_flags=( fBlocking == 0 ? 1 : 0) ;
	r = ioctlsocket (hSocket, FIONBIO, (unsigned long *)&sock_flags) ;
    }
#else
    {
	int r = -1;
	
	if( fBlocking )
	{
	}
	else
	{
#ifdef O_NONBLOCK
	    r = fcntl (hSocket, F_SETFL, O_NONBLOCK) ;
#else
	    r = fcntl (hSocket, F_SETFL, O_NDELAY) ;
#endif
	}
    }
#endif
    return r;
}

////////////////////////////////////////////////////////////////
static int ignoreErrno(int ierrno)
{
    switch (ierrno) {
#ifndef _WIN32
#if EINPROGRESS != EWOULDBLOCK
    case EINPROGRESS:
#endif
#endif
    case EWOULDBLOCK:
	return 1;
    default:
	return 0;
    }
    /* NOTREACHED */
}



////////////////////////////////////////////////////////////////
int SocketRecv( SOCKET hSocket, char * pBuffer, int cbBuffer )
{
    int cbRecv = recv( hSocket, pBuffer, cbBuffer, 0 );
    if( cbRecv < 0 )
    {
	if( ignoreErrno(SOCKETERRNO) )
	{
	    // Finalement, y'avait rien a lire
	    return 0;
	}

	// Probablement une deconnexion
	return -1;
    }

    // Renvoie la qtite lue
    return cbRecv;
}

////////////////////////////////////////////////////////////////
int SocketSend( SOCKET hSocket, char * pBuffer, int cbBuffer )
{
    int cbSent = send( hSocket, pBuffer, cbBuffer, 0 );
    if( cbSent < 0 )
    {
	if( ignoreErrno(SOCKETERRNO) )
	{
	    // On ne peut rien envoyer pour le moment
	    return 0;
	}

	// Probablement une deconnexion
	return -1;
    }

    // Renvoie la qtite ecrite
    return cbSent;
}

