// Gestion des evenements :
//  -rseau
//  -clavier
//  -timer
//  Compatible win32 console et unix
//
// Sylvain MARECHAL - sylvain.marechal1@libertysurf.fr
////////////////////////////////////////////////////////////////
#include <stdio.h>
#include "waitevt.h"


//////////////////////////////////////////////////////////////////
// Initialise les donnees interne du tableau aEvt
int InitExtractEvent( EvtRecord aEvt[], int cbEvt )
{
    // Initialisation avant tte chose
    // en particulier pour les sockets et les timer
    for( int i = 0; i < cbEvt; i ++ )
    {
	switch( aEvt[i].EvtType )
	{
	case EV_TYPE_KEYBOARD:
	    {
		break;
	    }
	case EV_TYPE_TIME:
	    {
		aEvt[i].EvtTime.ctInternal = GetClockCount();
		break;
	    }
	case EV_TYPE_SOCKET:
	    {
		aEvt[i].EvtSocket.hInternal = CreateSocketEvent();
		break;
	    }
	case EV_TYPE_SYSTEM:
	    {
		break;
	    }
	default:
	    {
		return ERROR_TYPE_UNKNOWN;
	    }
	}
    }
    return 0;
}

//////////////////////////////////////////////////////////////////
// Libere les ressources systemes allouees par 
// InitExtractEvent
int CloseExtractEvent( EvtRecord aEvt[], int cbEvt )
{
    for( int i = 0; i < cbEvt; i ++ )
    {
	switch( aEvt[i].EvtType )
	{
	case EV_TYPE_KEYBOARD:
	    {
		break;
	    }
	case EV_TYPE_TIME:
	    {
		break;
	    }
	case EV_TYPE_SOCKET:
	    {
		if( aEvt[i].EvtSocket.hInternal )
		{
		    CloseSocketEvent(aEvt[i].EvtSocket.hInternal);
		}
		break;
	    }
	case EV_TYPE_SYSTEM:
	    {
		break;
	    }
	default:
	    {
		return ERROR_TYPE_UNKNOWN;
	    }
	}
    }
    return 0;
}


//////////////////////////////////////////////////////////////////
// Transfere le tableau d'event en pEvtsMask de maniere
// qu'il soit utilisable par WaitForMultipleObjects() (win32)
// et selcet() (unix)
static int ExtractEventInitHandles( EvtRecord aEvt[], int cbEvt, EVTS_MASK * pEvtsMask )
{
    BOOL fKeyboardEventSet = FALSE;
    int iHandle = 0;
    static int s_iBegin = 0;

    // On met a jour les masks pour chaque EvtRecord
    //s_iBegin ++
    for( int it = s_iBegin; it < cbEvt + s_iBegin; it ++ )
    {
	int i = it % cbEvt;
	switch( aEvt[i].EvtType )
	{
	case EV_TYPE_KEYBOARD:
	    {
		// On ne met qu'une seule fois stdin, pour la premiere
		// touche
		if( fKeyboardEventSet == FALSE )
		{
		    SetKeyboardEventMask( &aEvt[i], pEvtsMask );
		    fKeyboardEventSet = TRUE;
		}
		break;
	    }
	case EV_TYPE_TIME:
	    {
		// Pas d'event lie a time
		break;
	    }
	case EV_TYPE_SOCKET:
	    {
		// On ajoute l'evenement lie a la socket
		if( SetSocketEventMask( &aEvt[i], pEvtsMask ) == SOCKET_ERROR )
		{
		    return ERROR_SOCKET;
		}
		break;
	    }
	case EV_TYPE_SYSTEM:
	    {
		// On ajoute les evenements systemes
		SetSystemEventMask( &aEvt[i], pEvtsMask );
		break;
	    }
	default:
	    {
		// tiens tiens ?
		return ERROR_TYPE_UNKNOWN;
	    }
	}
    }

    return iHandle;
}

//////////////////////////////////////////////////////////////////
// Recherche le plus petit des timeouts
// 
static DWORD FindTimeout( EvtRecord aEvt[], int cbEvt )
{
    DWORD dwTimeout;

    // Timeout infini au depart
    dwTimeout = INFINITE;

    // On remplit le tableau de handles
    for( int i = 0; i < cbEvt; i ++ )
    {
	switch( aEvt[i].EvtType )
	{
	case EV_TYPE_TIME:
	    {

		// On calcule le timeout qu'il reste
		DWORD Delay = GetClockCount() - aEvt[i].EvtTime.ctInternal;
		if( Delay > aEvt[i].EvtTime.timeout )
		{
		    Delay = aEvt[i].EvtTime.timeout;
		}
		if( dwTimeout == INFINITE )
		{
		    dwTimeout = Delay;
		}
		else
		{
		    dwTimeout = Delay < dwTimeout ? Delay : dwTimeout;
		}
		break;
	    }
	}
    }
    return dwTimeout;
}

    
//////////////////////////////////////////////////////////////////
// Rien	a faire : L'evenement systeme a simplement
// ete signale
static int ManageSystemEvent( EvtRecord * pEvt, EVTS_MASK * pEvtsMask )
{
    // Rien a faire
    return 0;
}


		


//////////////////////////////////////////////////////////////////
// Recherche quel evenement a ete signale, puis s'il est
// trouve, rempli la structure adequate
int ExtractEventFindEvent( EvtRecord aEvt[], int cbEvt, 
    EVTS_MASK * pEvtsMask, int aEventsSet[], int * pcbEventsSet  )
{
    int ret = 0;
    int nCharReaden = 0, nScanReaden = 0;
    int fKeyboardReaden = 0; int fKeyboardEventSetRead = 0;
    *pcbEventsSet = 0;

    // On boucle sur tous les evenements et on remplit le 
    // tableau aEventsSet[] avec les indices du tableau aEvt[]
    // qui sont signales.
    // La variable *pcbEventsSet memorise le nombre d'evenements
    // effectivement leves
    for( int i = 0; i < cbEvt; i ++ )
    {
	switch( aEvt[i].EvtType )
	{
	case EV_TYPE_KEYBOARD:
	    {
		// Touche du clavier
		// Dans ce cas, il faut rechercher de quel touche il s'agit
		// il y a en effet n evenements associe
		// On ne doit lire qu'une seule fois le clavier sous
		// peine de lire des touches en trop d'ou le fKeyboardEventSetRead
		if( fKeyboardEventSetRead == 0 )
		{
		    fKeyboardEventSetRead = 1;
		    if( IsEventSet( &aEvt[i], pEvtsMask ) )
		    {
			// On ne doit lire le clavier que la premiere fois.
			// -On memorise ensuite la touche lue
			if( ReadKeyboard(&nCharReaden, &nScanReaden ) > 0 )
			{
			    fKeyboardReaden = 1;
			}
		    }
		}

		// Si fKeyboardReaden est vrai, ca veut dire qu'on
		// a lue une touche, reste maintenant a savoir si
		// cette touche interesse l'utilisateur
		if( fKeyboardReaden )
		{
		    if( (/*nCharReaden == 0 &&*/
			    aEvt[i].EvtKeyboard.ScanCode == nScanReaden ) ||
			(aEvt[i].EvtKeyboard.AsciiChar == nCharReaden)		)
		    {
			aEventsSet[*pcbEventsSet] = i;
			(*pcbEventsSet) ++;
		    }
		}
		break;
	    }
	case EV_TYPE_SOCKET:
	    {
		// Evenement socket
		if( IsEventSet( &aEvt[i], pEvtsMask ) )
		{
		    ret = ManageSocketEvent( &aEvt[i], pEvtsMask );
		    aEventsSet[*pcbEventsSet] = i;
		    (*pcbEventsSet) ++;
		}
		break;
	    }
	case EV_TYPE_SYSTEM:
	    {
		// Evenement systeme
		if( IsEventSet( &aEvt[i], pEvtsMask ) )
		{
		    ret = ManageSystemEvent( &aEvt[i], pEvtsMask );
		    aEventsSet[*pcbEventsSet] = i;
		    (*pcbEventsSet) ++;
		}
		break;
	    }
	case EV_TYPE_TIME:
	    {
		unsigned long currentClock = GetClockCount();
		if( aEvt[i].EvtTime.ctInternal + aEvt[i].EvtTime.timeout <= currentClock )
		{
		    aEvt[i].EvtTime.ctInternal = currentClock;
		    aEventsSet[*pcbEventsSet] = i;
		    (*pcbEventsSet) ++;
		}
		break;
	    }
	default:
	    {
		// cas impossible
		ret = ERROR_TYPE_UNKNOWN;
		break;
	    }
	}
    }
    
    return ret;
}


//////////////////////////////////////////////////////////////////
int ExtractEvents( EvtRecord aEvt[], int cbEvt, int aEventsSet[], int * pcbEventsSet  )
{
    int ret = -1;
    EVTS_MASK EvtMask;

    // Initialisation
    InitEventMask( &EvtMask );

    // On set les events
    ret = ExtractEventInitHandles( aEvt, cbEvt, &EvtMask );
    if( ret < 0 ) return ret;

    
    // On attend qu'au moins un evenement soit signale
    *pcbEventsSet = 0;
    while( *pcbEventsSet == 0 )
    {
	// Recherche du plus petit temps
	DWORD dwTimeout = FindTimeout( aEvt, cbEvt );

	// Attente d'un evenement
	int iEvt = WaitEvent( &EvtMask, dwTimeout );

	// Extraction de l'evenement
	ret = ExtractEventFindEvent( aEvt, cbEvt, &EvtMask, aEventsSet, pcbEventsSet );
	
	// Erreur ?
	if( ret < 0 ) return ret;
    }
    
    return *pcbEventsSet;
}

