#ifndef WAITEVT_H
#define WAITEVT_H

#include "socket.h"

#ifdef __cplusplus
extern "C" {
#endif

#ifndef _WIN32
#ifndef HANDLE
#define HANDLE		int
#endif

#ifndef BOOL
#define BOOL		int
#endif

#ifndef FALSE
#define FALSE		0
#endif

#ifndef TRUE
#define TRUE		1
#endif

#ifndef INFINITE
#define INFINITE	(DWORD)-1
#endif

#ifndef NULL
#define NULL		0
#endif

#ifndef DWORD
#define DWORD		unsigned long
#endif

#else

#include <winsock2.h>
#define sleep(a)    Sleep(1000*(a))

#endif // ! _WIN32

#define ERROR_TYPE_UNKNOWN  -2
#define ERROR_WAIT_ERROR    -3
#define ERROR_SOCKET	    -4


// 4 types d'evenements possibles
typedef enum EVT_TYPE
{
    EV_TYPE_KEYBOARD,	// Appui sur une touche
    EV_TYPE_TIME,	// Timeout ecoule
    EV_TYPE_SOCKET,	// lire/ecrire/connect/accept/close sur une socket
    EV_TYPE_SYSTEM	// Handle systeme (WIn32)
} EVT_TYPE;

// Evenement clavier
typedef struct EVT_KEYBOARD
{
    int AsciiChar;
    int ScanCode;
} EVT_KEYBOARD;

// Evenement timer
typedef struct EVT_TIME
{
    int idTimer;
    unsigned long timeout;
    unsigned long ctInternal;
} EVT_TIME;

// Evenement socket
typedef struct EVT_SOCKET
{
    SOCKET hSocket;
    unsigned int MaskIn;
    unsigned int MaskOut;
    int Error;
    HANDLE hInternal;
} EVT_SOCKET;

// Evenement systeme
typedef struct EVT_SYSTEM
{
    HANDLE hHandle;
    int Mask;
} EVT_SYSTEM;


// Definition generique d'un evenement
typedef struct EvtRecord
{
    EVT_TYPE EvtType;
    union
    {
	EVT_KEYBOARD EvtKeyboard;
	EVT_TIME EvtTime;
	EVT_SOCKET EvtSocket;
	EVT_SYSTEM EvtSystem;
    };
} EvtRecord;


// Utilisation exterieure
// Initialisation des partie interne du tableau d'evenements
// a faire avant son utilisation
int InitExtractEvent( EvtRecord aEvt[], int cbEvt );

// Extraction d'evenements. Les index des evenements extraits
// sont dans le tableau aEventsSet
int ExtractEvents( EvtRecord aEvt[], int cbEvt, int aEventsSet[], int * pcbEventsSet  );

// Liberation des ressources
int CloseExtractEvent( EvtRecord aEvt[], int cbEvt );



/////////////////////////////////////////////////////////////////
// Utilisation interne
// differences unix/windows
//  Cette API est utilisee par waitevt.cpp
/////////////////////////////////////////////////////////////////
typedef struct EVTS_MASK
{
#ifdef _WIN32
    int nCount;					// Nombre de handles
    HANDLE aHandles[MAXIMUM_WAIT_OBJECTS];	// Tableau de handles pour WaitForMultipleObjects
    EvtRecord * aHandlesMapEvt[MAXIMUM_WAIT_OBJECTS];	// Correspondance handle event
    EvtRecord * pEvtSet;
#else
    fd_set MaskRInit;	// Init car c'
    fd_set MaskWInit;
    fd_set MaskEInit;
    fd_set MaskR;	// Sous unix, on utilise select() pour 
    fd_set MaskW;	// le clavier, les sockets et le timeout
    fd_set MaskE;
    int hi_fd;
#endif
} EVTS_MASK;


// Differences systemes unix/windows
// Tous les events
int InitEventMask( EVTS_MASK * pEvtsMask );
int WaitEvent( EVTS_MASK * pEvtsMask, unsigned long dwTimeout );
int IsEventSet( EvtRecord * pEvt, EVTS_MASK * pEvtsMask );

// Sockets
HANDLE CreateSocketEvent();
BOOL CloseSocketEvent( HANDLE hHandle );
int SetSocketEventMask( EvtRecord * pEvt, EVTS_MASK * pEvtsMask );
int ManageSocketEvent( EvtRecord * pEvt, EVTS_MASK * pEvtsMask );

// Timer
unsigned long GetClockCount();

// Clavier
int ReadKeyboard( int * pAsciiChar, int *pwVirtualScanCode);
int SetKeyboardEventMask( EvtRecord * pEvt, EVTS_MASK * pEvtsMask );

// System
int SetSystemEventMask( EvtRecord * pEvt, EVTS_MASK * pEvtsMask );



#ifdef __cplusplus
}
#endif

#endif //WAITEVT_H

