////////////////////////////////////////////////////////////////
// Gestion des evenements pour unix utilisee par waitevt.cpp
//  -reseau
//  -clavier
//  -timer
// Repose sur select()
//
// Sylvain MARECHAL - sylvain.marechal1@libertysurf.fr
////////////////////////////////////////////////////////////////
#include <stdio.h>
#include <sys/time.h>
#include "waitevt.h"

//////////////////////////////////////////////////////////////////
// Rien a faire pour unix
HANDLE CreateSocketEvent()
{
    return 0;
}

//////////////////////////////////////////////////////////////////
// Rien a faire pour unix
BOOL CloseSocketEvent( HANDLE hHandle )
{
    return 1;
}


//////////////////////////////////////////////////////////////////
// Set le mask
int SetSocketEventMask( EvtRecord * pEvt, EVTS_MASK * pEvtsMask )
{
    if( pEvt->EvtSocket.MaskIn & FD_READ || 
	pEvt->EvtSocket.MaskIn & FD_CLOSE ||
	pEvt->EvtSocket.MaskIn & FD_ACCEPT  )
    {
	// Lecture, fermeture ou attente d'acceptation
	FD_SET( pEvt->EvtSocket.hSocket, &(pEvtsMask->MaskRInit) );
    }
    if( pEvt->EvtSocket.MaskIn & FD_WRITE )
    {
	// Demande d'ecriture
	FD_SET( pEvt->EvtSocket.hSocket, &(pEvtsMask->MaskWInit) );
    }
    if( pEvt->EvtSocket.MaskIn & FD_CONNECT )
    {
	// Attente de connexion
	FD_SET( pEvt->EvtSocket.hSocket, &(pEvtsMask->MaskWInit) );
	FD_SET( pEvt->EvtSocket.hSocket, &(pEvtsMask->MaskEInit) );
    }
    
    // On met a jour le handle le plus grand
    pEvtsMask->hi_fd = (pEvtsMask->hi_fd < pEvt->EvtSocket.hSocket ? 
	pEvt->EvtSocket.hSocket : pEvtsMask->hi_fd);
    return 0;
}

//////////////////////////////////////////////////////////////////
// Remplit MaskOut qui correspond aux evenements
// reseaux en sorti
int ManageSocketEvent( EvtRecord * pEvt, EVTS_MASK * pEvtsMask )
{
    if( FD_ISSET( pEvt->EvtSocket.hSocket, &(pEvtsMask->MaskR) ) )
    {
	if( pEvt->EvtSocket.MaskIn & FD_READ )
	{
	    // On voulait lire de la donnee, et c'est possible
	    pEvt->EvtSocket.MaskOut = FD_READ;
	}
	if( pEvt->EvtSocket.MaskIn & FD_ACCEPT )
	{
	    // On attendait une connexion et y'en a une de prete
	    pEvt->EvtSocket.MaskOut = FD_ACCEPT;
	}	    
    }
    if( FD_ISSET( pEvt->EvtSocket.hSocket, &(pEvtsMask->MaskW) ) )
    {
	if( pEvt->EvtSocket.MaskIn & FD_WRITE )
	{
	    // On voulait ecrire et c'est possible
	    pEvt->EvtSocket.MaskOut = FD_WRITE;
	}
	if( pEvt->EvtSocket.MaskIn & FD_CONNECT )
	{
	    // On voulait se connecter et la connexion
	    // est realisee
	    pEvt->EvtSocket.MaskOut = FD_CONNECT;
	}
    }
    if( FD_ISSET( pEvt->EvtSocket.hSocket, &(pEvtsMask->MaskE) ) )
    {
	pEvt->EvtSocket.Error = errno;
	pEvt->EvtSocket.MaskOut = FD_CONNECT;
    }
    return 0;
}

//////////////////////////////////////////////////////////////////
// RAZ
int InitEventMask( EVTS_MASK * pEvtsMask )
{
    FD_ZERO( &(pEvtsMask->MaskRInit) );
    FD_ZERO( &(pEvtsMask->MaskWInit) );
    FD_ZERO( &(pEvtsMask->MaskEInit) );
    pEvtsMask->hi_fd = 0;
    return 0;
}

//////////////////////////////////////////////////////////////////
// 
int IsEventSet( EvtRecord * pEvt, EVTS_MASK * pEvtsMask )
{
    int ret = 0;
    switch( pEvt->EvtType )
    {
	case EV_TYPE_KEYBOARD:
	    {
		ret = FD_ISSET( 0, &(pEvtsMask->MaskR) );
		break;
	    }
	case EV_TYPE_SOCKET:
	    {
		ret = FD_ISSET( pEvt->EvtSocket.hSocket, &(pEvtsMask->MaskR) ) ||
		    FD_ISSET(pEvt->EvtSocket.hSocket, &(pEvtsMask->MaskW) ) ||
		    FD_ISSET( pEvt->EvtSocket.hSocket, &(pEvtsMask->MaskE) );
		break;
	    }
    }
    return ret;
}

//////////////////////////////////////////////////////////////////
// Attente d'un evenement
// 
int WaitEvent( EVTS_MASK * pEvtsMask, unsigned long dwTimeout )
{
    struct timeval max_delay, *pTZ;

    // On transforme le timeout pour select()
    if( dwTimeout == INFINITE )
    {
	pTZ = NULL ;
    }
    else
    {
	max_delay.tv_sec = dwTimeout/1000;
	max_delay.tv_usec = (dwTimeout%1000)*1000;
	pTZ = &max_delay ;
    }

    // On copie les masks qu'on a initialise car 
    // le mask est modifie par select()
    pEvtsMask->MaskR = pEvtsMask->MaskRInit;
    pEvtsMask->MaskW = pEvtsMask->MaskWInit;
    pEvtsMask->MaskE = pEvtsMask->MaskEInit;
    
    // select()
    int retselect = select( pEvtsMask->hi_fd+1, 
			    &(pEvtsMask->MaskR), 
			    &(pEvtsMask->MaskW), 
			    &(pEvtsMask->MaskE), pTZ );
    return retselect;
}

//////////////////////////////////////////////////////////////////
// Remvoie de l'heure relative en millisecondes
unsigned long GetClockCount()
{
    static struct timeval ClockCount;
    struct timeval Now;
    static long diff;
    
    if( ClockCount.tv_sec != 0 )
    {
        gettimeofday(&Now, NULL);
	diff += (Now.tv_sec - ClockCount.tv_sec) * 1000L;
	diff += (Now.tv_usec - ClockCount.tv_usec) / 1000L;
	ClockCount.tv_sec = Now.tv_sec ;
	ClockCount.tv_usec = Now.tv_usec ;
        return diff;
    }
    else
    {
        gettimeofday(&ClockCount, NULL);
	diff = 0 ;
    }
    return 0;
}

//////////////////////////////////////////////////////////////////
// Lecture d'une touche (code ascii et scan code)
int ReadKeyboard( int * pAsciiChar, int *pwVirtualScanCode)
{
    char ch;

    *pAsciiChar = 0;
    *pwVirtualScanCode = 0;
    int cb = read(0,&ch,1);
    if( cb > 0 )
    {
	*pAsciiChar = ch;
    }
    return cb;
}


//////////////////////////////////////////////////////////////////
// Set le mask pour le clavier
int SetKeyboardEventMask( EvtRecord * pEvt, EVTS_MASK * pEvtsMask )
{
    FD_SET( 0, &(pEvtsMask->MaskRInit) );
    return 0;
}

//////////////////////////////////////////////////////////////////
// Set le mask system
int SetSystemEventMask( EvtRecord * pEvt, EVTS_MASK * pEvtsMask )
{
    FD_SET( pEvt->EvtSystem.hHandle, &(pEvtsMask->MaskRInit) );
    return 0;
}

