////////////////////////////////////////////////////////////////
// Gestion des evenements pour Win32 utilisee par waitevt.cpp
//  -rseau
//  -clavier
//  -timer
// Repose sur WaitForMultipleObjects()
//
// Sylvain MARECHAL - sylvain.marechal1@libertysurf.fr
////////////////////////////////////////////////////////////////
#include <winsock2.h>
#include <stdio.h>
#include "waitevt.h"


//////////////////////////////////////////////////////////////////
// Creation d'un event pour WSAEventSelect()
HANDLE CreateSocketEvent()
{
    return WSACreateEvent();
}

//////////////////////////////////////////////////////////////////
// Fermure de l'event de WSAEventSelect()
BOOL CloseSocketEvent( HANDLE hHandle )
{
    return WSACloseEvent(hHandle);
}


//////////////////////////////////////////////////////////////////
// Set le mask
int SetSocketEventMask( EvtRecord * pEvt, EVTS_MASK * pEvtsMask )
{
    int iHandle = pEvtsMask->nCount;
    
    // On mappe
    pEvtsMask->aHandlesMapEvt[iHandle] = pEvt;
    
    // WSAEventSelect
    int ret = WSAEventSelect( pEvt->EvtSocket.hSocket, 
	pEvt->EvtSocket.hInternal, 
	pEvt->EvtSocket.MaskIn );
    if( ret != SOCKET_ERROR )
    {
	pEvtsMask->aHandles[iHandle] = pEvt->EvtSocket.hInternal;
	pEvtsMask->nCount ++;
    }
    return ret;
}

//////////////////////////////////////////////////////////////////
// Remplit MaskOut qui correspond aux evenements
// reseaux en sorti
int ManageSocketEvent( EvtRecord * pEvt, EVTS_MASK * /*pEvtsMask*/ )
{
    WSANETWORKEVENTS NetworkEvents;
    if (SOCKET_ERROR == WSAEnumNetworkEvents( pEvt->EvtSocket.hSocket, pEvt->EvtSocket.hInternal, &NetworkEvents)) 
    { 
printf("WSAEnumNetworkEvent: %d lNetworkEvent %X\n",  WSAGetLastError(), &NetworkEvents.lNetworkEvents); 
	return -1;
    }
    pEvt->EvtSocket.MaskOut = NetworkEvents.lNetworkEvents;
    pEvt->EvtSocket.Error = NetworkEvents.iErrorCode[NetworkEvents.lNetworkEvents];
    return 0;
}

//////////////////////////////////////////////////////////////////
// RAZ
int InitEventMask( EVTS_MASK * pEvtsMask )
{
    memset( pEvtsMask, 0, sizeof(EVTS_MASK) );
    return 0;
}


//////////////////////////////////////////////////////////////////
// 
int IsEventSet( EvtRecord * pEvt, EVTS_MASK * pEvtsMask )
{
    return pEvt == pEvtsMask->pEvtSet;
}

//////////////////////////////////////////////////////////////////
// Attente d'un evenement
int WaitEvent( EVTS_MASK * pEvtsMask, unsigned long dwTimeout )
{
    int ret;
    
    pEvtsMask->pEvtSet = NULL;
    DWORD dwWait = WaitForMultipleObjects( pEvtsMask->nCount, pEvtsMask->aHandles, FALSE, dwTimeout );
    if( (int)dwWait >= WAIT_OBJECT_0 && (int)dwWait < WAIT_OBJECT_0 + pEvtsMask->nCount )
    {
	int iHandle = dwWait - WAIT_OBJECT_0;
	pEvtsMask->pEvtSet = pEvtsMask->aHandlesMapEvt[iHandle];
    }
    else if( dwWait == WAIT_TIMEOUT )
    {
	// Timeout
	ret = 0;
    }
    else
    {
	// Erreur
	printf( "ERROR (dwWait=%d) (error=%d)\n", dwWait, GetLastError() );
	ret = -1;
    }

    return ret;
}


//////////////////////////////////////////////////////////////////
// Remvoie de l'heure relative en millisecondes
unsigned long GetClockCount()
{
    return GetTickCount();
}


//////////////////////////////////////////////////////////////////
// Lecture d'une touche (code ascii et scan code)
int ReadKeyboard( int * pAsciiChar, int *pwVirtualScanCode)
{
    INPUT_RECORD InputRecord;
    DWORD nLength = 1;
    DWORD dwNumberOfEventsRead;

    *pwVirtualScanCode = 0;
    *pAsciiChar = 0;

    if( ! ReadConsoleInput( GetStdHandle(STD_INPUT_HANDLE), 
	&InputRecord, nLength, &dwNumberOfEventsRead  ) )
    {
	return -1;
    }
    if( dwNumberOfEventsRead < 1 )
    {
	return 0;
    }
    if( InputRecord.EventType != KEY_EVENT )
    {
	return 0;
    }
   /* printf( "bKeyDown=%d\n"
	    "wRepeatCount=%d\n"
	    "wVirtualKeyCode=%d\n"
	    "wVirtualScanCode=%d\n"
	    "uChar=%d\n"
	    "dwControlKeyState=%d\n",
	    InputRecord.Event.KeyEvent.bKeyDown,
	    InputRecord.Event.KeyEvent.wRepeatCount,
	    InputRecord.Event.KeyEvent.wVirtualKeyCode,
	    InputRecord.Event.KeyEvent.wVirtualScanCode,
	    InputRecord.Event.KeyEvent.uChar.AsciiChar,
	    InputRecord.Event.KeyEvent.dwControlKeyState );*/

    *pAsciiChar = InputRecord.Event.KeyEvent.uChar.AsciiChar;
    *pwVirtualScanCode = InputRecord.Event.KeyEvent.wVirtualScanCode;
    return 1;
}

//////////////////////////////////////////////////////////////////
// Set le mask pour le clavier
int SetKeyboardEventMask( EvtRecord * pEvt, EVTS_MASK * pEvtsMask )
{
    int iHandle = pEvtsMask->nCount;
    pEvtsMask->aHandlesMapEvt[iHandle] = pEvt;
    pEvtsMask->aHandles[iHandle] = GetStdHandle(STD_INPUT_HANDLE);
    pEvtsMask->nCount ++;
    return 0;
}

//////////////////////////////////////////////////////////////////
// Set le mask system
int SetSystemEventMask( EvtRecord * pEvt, EVTS_MASK * pEvtsMask )
{
    int iHandle = pEvtsMask->nCount;
    pEvtsMask->aHandlesMapEvt[iHandle] = pEvt;
    pEvtsMask->aHandles[iHandle] = pEvt->EvtSystem.hHandle;
    pEvtsMask->nCount ++;
    return 0;
}

